////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                //
// HEL Library (c) 2003, 2004 by Peter Schraut                                                    //
// http://www.console-dev.de                                                                      //
//                                                                                                //
// ---------------------------------------------------------------------------------------------- //
//                                                                                                //
// HEL Library is distributed as freeware.                                                        //
//                                                                                                //  
// You are free to use HEL Library as part of your program for any purpose including freeware,    //
// commercial and shareware programs, provided some credit is given to the author.                //
//                                                                                                // 
// The origin of this software must not be misrepresented; you must not claim your authorship.    //
// All redistributions must retain the original copyright notice and web site addresses.          //
//                                                                                                // 
// Commercial redistribution of the library is allowed only with an explicit written              // 
// permission from the author.                                                                    // 
//                                                                                                // 
// If you finished a commercial game using HEL library, the author would like to be informed      // 
// in what game it is used.                                                                       // 
//                                                                                                // 
// This software is provided 'as-is', without warranty of any kind, either expressed or implied.  // 
// In no event shall the author be held liable for any damages arising from the use of this       //
// software.                                                                                      // 
//                                                                                                // 
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef HEL_H
#define HEL_H

#include <mygba.h>
#include <stdarg.h>
#include <malloc.h>

//
// ---[ HEL version defines ]-----------------------------------------------------------------------
//

#define HEL_CURRENT_VERSION_MAJOR   1
#define HEL_CURRENT_VERSION_MINOR   2
#define HEL_CURRENT_VERSION_STRING  "1.2"

//
// ---[ Version checking macros for HAM ]-----------------------------------------------------------
//

//
// EQ=Equal, NE=NotEqual, GT=GreaterThan, GTE=GreaterThanEqual, LE=LessEqual, LT=LessThan
//
#define CURRENT_HAM_VERSION_EQ(major, minor)   (HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR >= (minor))
#define CURRENT_HAM_VERSION_NE(major, minor)   (HAM_CURRENT_VERSION_MAJOR != (major) || HAM_CURRENT_VERSION_MINOR != (minor))
#define CURRENT_HAM_VERSION_GT(major, minor)   ((HAM_CURRENT_VERSION_MAJOR > (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR > (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_GTE(major, minor)  ((HAM_CURRENT_VERSION_MAJOR > (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR >= (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_LE(major, minor)   ((HAM_CURRENT_VERSION_MAJOR < (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR <= (minor))?TRUE:FALSE)
#define CURRENT_HAM_VERSION_LT(major, minor)   ((HAM_CURRENT_VERSION_MAJOR < (major))?TRUE:(HAM_CURRENT_VERSION_MAJOR == (major) && HAM_CURRENT_VERSION_MINOR < (minor))?TRUE:FALSE)


//
// ---[ General Typedef's ]-------------------------------------------------------------------------
//

typedef struct
{
   s8 X, Y;
}TPoint8;

typedef struct
{
   u8 X, Y;
}TPointU8;

typedef struct
{
   s16 X, Y;
}TPoint16;

typedef struct
{
   u16 X, Y;
}TPointU16;

typedef struct
{
   s32 X, Y;
}TPoint32;

typedef struct
{
   u32 X, Y;
}TPointU32;

typedef struct
{
   s8 Left;
   s8 Right;
   s8 Top;
   s8 Bottom;
}TRect8;

typedef struct
{
   s16 Left;
   s16 Right;
   s16 Top;
   s16 Bottom;
}TRect16;

typedef struct
{
   s32 Left;
   s32 Right;
   s32 Top;
   s32 Bottom;
}TRect32;



//
// ---[ Defines ]-----------------------------------------------------------------------------------
//

#define BIT(n)                         (1<<(n))
#define BIT_NOT(n)                     (~BIT(n))
#define BIT_IS_SET(v,b)                ((((v) >> (b)) & 1))
#define SHR(v,n)                       ((v) >> (n))
#define SHL(v,n)                       ((v) << (n))
#define MAX(a,b)                       (((a)>(b))?(a):(b))
#define MIN(a,b)                       (((a)<(b))?(a):(b))
#define SGN(v)                         (((v) < 0)?-1:(((v) > 0)?(+1):(0)))
#define ABS(v)                         (((v) > 0)?(v):(-(v)))
#define ASSIGNED(p)                    (NULL != (p))
#define ACCESS_U8(v)                   *(volatile u8*)(v)
#define ACCESS_U16(v)                  *(volatile u16*)(v)
#define ACCESS_U32(v)                  *(volatile u32*)(v)
#define RANDOM(n)                      (rand()%(n))
#define SIN(n)                         (SIN_SFP32[(n)%360])
#define COS(n)                         (COS_SFP32[(n)%360])
#define ATTR_ALIGNED(n)                __attribute__ ((aligned(n)))
#define ATTR_PACKED                    __attribute__ ((packed))
#define ATTR_SECTION(name)             __attribute__ ((section(name)))

//
// Use it like:
//  TMyStruct *p;
//      p = NEW(TMyStruct, 10); // allocs memory for 10 TMyStruct's
//
// use HAM's malloc wrapper if HAM version >= 2.80 and HAM_DEBUGGER is enabled
#if CURRENT_HAM_VERSION_GTE(2,80) && HAM_DEBUGGER
   #define NEW(struct_, count)         (struct_*)ham_Malloc((count)*sizeof(struct_))
#else
   #define NEW(struct_, count)         (struct_*)malloc((count)*sizeof(struct_))
#endif

#define FREE(p)                        { if(ASSIGNED(p)){ free(p); (p)=NULL; } }
#define EXTERN_DATA8(name,size)        extern const u8 name[size];
#define EXTERN_DATA16(name,size)       extern const u16 name[size];
#define EXTERN_DATA32(name,size)       extern const u32 name[size];



// VBAText stuff does -NOT- work on hardware!!!
#ifdef HAM_DEBUGGER
   #define VBAOUT(txt, args...)             ham_VBAText(txt,##args);
#else
   #define VBAOUT(txt, args...)
#endif


#define SET_POINT(P, X_, Y_)        \
   {                                \
      (P)->X = X_; (P)->Y = Y_;     \
   }

#define SET_RECT(P, Left_, Top_, Right_, Bottom_)  \
   {                                               \
      (P)->Left=Left_;   (P)->Top=Top_;            \
      (P)->Right=Right_;  (P)->Bottom=Bottom_;     \
   }




#ifndef __cplusplus
   #define false     0
   #define true      (!false)
   
   #ifndef bool
      #define bool    u8
   #endif // bool
#endif


// Can be used for TOOL_DMA0/1/2/3_SET_EX
#define DMA_INCREMENT       (0)
#define DMA_DECREMENT       (1)
#define DMA_NOCHANGE        (2)
#define DMA_REPEAT_ON       (1)
#define DMA_REPEAT_OFF      (0)
#define DMA_IRQ_ON          (1)
#define DMA_IRQ_OFF         (0)


#if CURRENT_HAM_VERSION_LT(2,80)
   // can be used with ham_InitMapEmptySet(..)
   // normal map sizes
   #define MAP_SIZE_32X32        0
   #define MAP_SIZE_64X32        1
   #define MAP_SIZE_32X64        2
   #define MAP_SIZE_64X64        3
   // rotationmap sizes
   #define MAP_SIZE_ROT_16X16    0
   #define MAP_SIZE_ROT_32X32    1
   #define MAP_SIZE_ROT_64X64    2
   #define MAP_SIZE_ROT_128X128  3
#endif


#if CURRENT_HAM_VERSION_LT(2,80)
   #define OBJ_COLORS_16         0
   #define OBJ_COLORS_256        1
#endif


// Helper defines. Can be used for hel_Splash(..)
#define SPLASH_TYPE_BLACK        0
#define SPLASH_TYPE_WHITE        1


// Helper defines for hel_FadePal(..)
#define FADE_PALETTE_BG          0
#define FADE_PALETTE_OBJ         1
#define FADE_COLORS_16           0
#define FADE_COLORS_256          1


// Helper defines for windowing
#define WIN_ALL                  (WIN_BG0 | WIN_BG1 | WIN_BG2 | WIN_BG3 | WIN_OBJ)
#define WIN_NONE                 (0)
// specify 0 or 1 to whether exclude or include window
#define WIN_SELECT(bg0, bg1, bg2, bg3, obj)   \
   ( (WIN_BG0*(bg0)) | (WIN_BG1*(bg1)) | (WIN_BG2*(bg2)) | (WIN_BG3*(bg3)) | (WIN_OBJ*(obj)) )


// Conversion helpers for the Map-System parallax functions
#define PARALLAX_RATIO_ONE                0xffff
#define PARALLAX_FLOAT_TO_RATIO(value)    CONV_FLOAT_TO_SFP32(value)
#define PARALLAX_RATIO_TO_FLOAT(value)    (float)((value)/(float)PARALLAX_RATIO_ONE)
#define PARALLAX_RATIO_TO_INT(value)      CONV_SFP32_TO_S16(value)


//
// ---[ Helper macros ]-----------------------------------------------------------------------------
//

//
// background creation macro for gfx2gba nameconventions.
// for example:
//             H_CREATE_BG(Background, 240, 160, 1, 0, 2, 0);
//
// Example creates a 240x160 pixel background on layer1, using 16 colors
// and prio2, no rotation map.
// Arrays are called Background_Tiles and Background_Map
//
#define H_BG_CREATE(ArrayName, Width, Height, BgNo, ColMode, Prio, RotationMap) \
    {                                                                           \
        u8 MapSize=0;                                                           \
        map_fragment_info *pMapInfo=NULL;                                       \
                                                                                \
        if(!RotationMap)                                                        \
        {                                                                       \
            if(Width <= 256 && Height <= 256)       MapSize=0;                  \
            else if(Width <= 512  && Height <= 256) MapSize=1;                  \
            else if(Width <= 256 && Height <= 512)  MapSize=2;                  \
            else if(Width <= 512 && Height <= 512)  MapSize=3;                  \
        }                                                                       \
        else                                                                    \
        {                                                                       \
            if(Width <= 128 && Height <= 128)        MapSize=0;                 \
            else if(Width <= 256  && Height <= 256)  MapSize=1;                 \
            else if(Width <= 512  && Height <= 512)  MapSize=2;                 \
            else if(Width <= 1024 && Height <= 1024) MapSize=3;                 \
        }                                                                       \
                                                                                \
        ham_bg[BgNo].ti = ham_InitTileSet((void*)ArrayName##_Tiles,             \
                            SIZEOF_16BIT(ArrayName##_Tiles), ColMode, 1);       \
                                                                                \
        ham_bg[BgNo].mi = ham_InitMapEmptySet(MapSize, RotationMap);            \
                                                                                \
        pMapInfo = ham_InitMapFragment((void*)ArrayName##_Map, Width/8,         \
                        Height/8,  0, 0, Width/8, Height/8, RotationMap);       \
                                                                                \
        ham_InsertMapFragment(pMapInfo, BgNo, 0, 0);                            \
        ham_InitBg(BgNo, FALSE, Prio, FALSE);                                   \
        pMapInfo = NULL;                                                        \
    }


#define H_BG_DEINIT_SELECT(bg0, bg1, bg2, bg3)                                  \
    {                                                                           \
        if(bg0) { ham_SetBgXY(0, 0, 0); ham_DeInitBg(0); }                      \
        if(bg1) { ham_SetBgXY(1, 0, 0); ham_DeInitBg(1); }                      \
        if(bg2) { ham_SetBgXY(2, 0, 0); ham_DeInitBg(2); }                      \
        if(bg3) { ham_SetBgXY(3, 0, 0); ham_DeInitBg(3); }                      \
    }


#define H_BG_VISIBLE_SELECT(bg0, bg1, bg2, bg3)                                 \
    {                                                                           \
        if(bg0) ham_SetBgVisible(0, TRUE);                                      \
        if(bg1) ham_SetBgVisible(1, TRUE);                                      \
        if(bg2) ham_SetBgVisible(2, TRUE);                                      \
        if(bg3) ham_SetBgVisible(3, TRUE);                                      \
    }


#define H_BG_INVISIBLE_SELECT(bg0, bg1, bg2, bg3)                               \
    {                                                                           \
        if(bg0) ham_SetBgVisible(0, FALSE);                                     \
        if(bg1) ham_SetBgVisible(1, FALSE);                                     \
        if(bg2) ham_SetBgVisible(2, FALSE);                                     \
        if(bg3) ham_SetBgVisible(3, FALSE);                                     \
    }


#define H_FULL_DARKEN                                                           \
    {                                                                           \
        ham_SetFxMode(FX_LAYER_SELECT(1,1,1,1,1,1),                             \
                      FX_LAYER_SELECT(1,1,1,1,1,1),                             \
                      FX_MODE_DARKEN);                                          \
                                                                                \
        ham_SetFxBrightnessLevel(16);                                           \
    }


#define H_WAIT_VSYNC { while((volatile u16)REG_VCOUNT != 160){} }




#ifdef __cplusplus
   extern "C" {
#endif

//
// ---[ Forward declarations ]----------------------------------------------------------------------
//
struct _TMapScrollInfo; 

//
// ---[ Map functions ]-----------------------------------------------------------------------------
//

typedef void (*PMapNotifyFunc)(struct _TMapScrollInfo *pMapInfo, u32 X, u32 Y);

typedef struct
{
   PMapNotifyFunc pOnTopRowChanged;
   PMapNotifyFunc pOnLeftColumnChanged;
}TMapNotifier, *PMapNotifier;

typedef struct _TMapParallaxInfo
{
   TPoint32 Ratio;
   TPoint32 CurrentAmount;
}TMapParallaxInfo, *PMapParallaxInfo;

typedef struct _TMapScrollInfo
{
   u8 BgNo;
   TPoint16 Size;
   void *pData;
   TPoint16 Position;
   TPoint8 Runner;
   u8 BoundsCheck;
   TMapNotifier MapRedrawNotifiers;
   u8 UseParallax; // added in v1.2
   TMapParallaxInfo ParallaxInfo; // added in v1.2
}TMapScrollInfo, *PMapScrollInfo;

extern TMapScrollInfo hel_MapInit(u8 BgNo, u16 TilesX, u16 TilesY, u8 Rotation, void *pMapData);
extern void hel_MapDeInit(TMapScrollInfo *pMapInfo);
extern TMapScrollInfo hel_MapInitEx(u8 BgNo, u16 TilesX, u16 TilesY, u8 Rotation, u8 MapSize, void *pMapData);
extern void hel_MapSetCallbacks(TMapScrollInfo *pMapInfo, PMapNotifyFunc pOnTopRowChanged, PMapNotifyFunc pOnLeftColumnChanged);
extern u8 hel_MapScrollBy(TMapScrollInfo *pMapInfo, s32 DeltaX, s32 DeltaY);
extern u8 hel_MapIsBoundsCheckEnabled(TMapScrollInfo *pMapInfo);
extern void hel_MapSetBoundsCheck(TMapScrollInfo *pMapInfo, u8 Value);
extern void hel_MapSetPosition(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
extern void hel_MapSetPositionInPixel(TMapScrollInfo *pMapInfo, u32 X, u32 Y);
extern TPoint32 hel_MapGetPosition(TMapScrollInfo *pMapInfo);
extern TPoint32 hel_MapGetPositionInPixel(TMapScrollInfo *pMapInfo);
extern void hel_MapRedraw(TMapScrollInfo *pMapInfo);
extern void hel_MapBatchScrollBy(TMapScrollInfo *pMapInfo, s32 DeltaX, s32 DeltaY, int Count);
extern void hel_MapSetParallaxRatio(TMapScrollInfo *pMapInfo, FIXED RatioX, FIXED RatioY);
extern void hel_MapSetParallax(TMapScrollInfo *pMapInfo, u8 Value);

//
// ---[ Map Camera functions ]----------------------------------------------------------------------
//

#define MAP_CAM_NEXTKEYPOINT    0x11
#define MAP_CAM_LASTKEYREACHED  0x12

typedef TPoint16 TMapCamKeyPoint;
typedef TMapCamKeyPoint *PMapCamKeyPoint;


typedef struct _TMapCamScrollInfo
{
   TMapCamKeyPoint *pKeyPoints;
   u16 nKeyPoints;
   u16 CurKeyPoint;
}TMapCamScrollInfo, *PMapCamScrollInfo;

extern u8 hel_MapScrollTo(TMapScrollInfo *pMapInfo, TMapCamScrollInfo *pMapCamInfo, u32 SpeedX, u32 SpeedY);



//
// ---[ Text functions ]----------------------------------------------------------------------------
//

typedef struct
{
   u8 BgNo;
   char *CharOrder;
   u8 CharIndex[256];
   // fix in 0.9
   u8 ColMode;
   u8 PalNo;
}TCustomTextInfo, *PCustomTextInfo;

//extern void hel_CustomTextInit(TCustomTextInfo *pTextInfo, void *pSrcTiles, u16 SizeU16, u8 ColMode, u8 MapSize, u8 MapRot);
extern void hel_CustomTextInit(TCustomTextInfo *pTextInfo, void *pSrcTiles, u16 SizeU16, u8 ColMode, u8 PalNo, u8 MapSize, u8 MapRot);
extern void hel_CustomTextClear(TCustomTextInfo *pTextInfo);
extern void hel_CustomTextClearRow(TCustomTextInfo *pTextInfo, u32 y);
extern void hel_CustomTextPrint(TCustomTextInfo *pTextInfo, u32 x, u32 y, char *pText);
extern void hel_CustomTextPrintFmt(TCustomTextInfo *pTextInfo, u32 x, u32 y, char *pFmt, ...);
extern u8 hel_CustomTextType(TCustomTextInfo *pTextInfo, u32 x, u32 y, u8 Delay, u16 Wait, char *pText);

//
// ---[ Pad functions ]-----------------------------------------------------------------------------
//

typedef struct _TPadButtons
{
   u8 Left:1;
   u8 Right:1;
   u8 Down:1;
   u8 Up:1;
   u8 Start:1;
   u8 Select:1;
   u8 A:1;
   u8 B:1;
   u8 L:1;
   u8 R:1;
   u8 None:1;
}TPadButtons, *PPadButtons;

typedef struct
{
   TPadButtons Pressed;
   TPadButtons Held;
}TPad, *PPad;

extern TPad hel_Pad; // pad variable
extern void hel_PadInit(void);
extern void hel_PadReset(void);
extern void hel_PadReadState(void);


//
// ---[ Palette functions ]-------------------------------------------------------------------------
//

extern void hel_PalBgInvert16(u8 PaletteBank);
extern void hel_PalBgInvert256(void);
extern void hel_PalBgInvertEx(u8 StartIndex, u8 EndIndex);
extern void hel_PalBgClear16(u8 PaletteBank);
extern void hel_PalBgClear256(void);
extern void hel_PalBgClearEx(u8 StartIndex, u8 EndIndex, u8 Red, u8 Green, u8 Blue);


//
// ---[ Obj functions ]-----------------------------------------------------------------------------
//

extern u8 hel_ObjExists(u8 ObjNo);
extern u8 hel_ObjCreate(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, u16 ColMode, u16 PalNo, u16 Mosaic, u16 HFlip, u16 VFlip, u16 Prio, u16 DblSize, s16 X, s16 Y);
extern u8 hel_ObjCreate256(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, s16 X, s16 Y);
extern u8 hel_ObjCreate16(void* pSrc, u16 ObjShape, u16 ObjSize, u16 ObjMode, u16 PalNo, s16 X, s16 Y);
extern inline void hel_ObjUpdateInOAM(u8 ObjNo);
extern u8 hel_ObjIsVFlip(u8 ObjNo);
extern u8 hel_ObjIsHFlip(u8 ObjNo);
extern u8 hel_ObjIsVisible(u8 ObjNo);
extern u8 hel_ObjIsMosaic(u8 ObjNo);
extern u8 hel_ObjIsRotScale(u8 ObjNo);
extern u8 hel_ObjGetColorMode(u8 ObjNo);
extern void hel_ObjBringToFront(u8 ObjNo);
extern u8 hel_ObjGetFirst(void);


//
// ---[ DMA functions ]-----------------------------------------------------------------------------
//

extern void hel_DmaCopy16(void *pDest, const void *pSrc, u32 SizeIn16BitWords);
extern void hel_DmaCopy32(void *pDest, const void *pSrc, u32 SizeIn32BitWords);
extern void hel_DmaSet16(void *pDest, u16 SrcValue, u32 SizeIn16BitWords);
extern void hel_DmaSet32(void *pDest, u32 SrcValue, u32 SizeIn32BitWords);


//
// ---[ Sprite functions ]--------------------------------------------------------------------------
//

typedef struct _TSprite
{
   u8 ObjNo;
   s16 X,Y;
}TSprite, *PSprite;


//
// ---[ Splash functions ]--------------------------------------------------------------------------
//

extern void hel_Splash(void *pBitmap, void *pPalette, u8 SplashTypeIn, u8 SplashTypeOut, u8 FadeInSpeed, u8 FadeOutSpeed, u8 WaitFrames, u8 AfterWaitFrames);


//
// ---[ Fade functions ]----------------------------------------------------------------------------
//

typedef struct _TFadeInfo
{
   void *pPalette;
   u8 PalMode;
   u8 ColMode;
   u8 PalNo;
   u8 CurrentAmount;
}TFadeInfo, *PFadeInfo;


extern TFadeInfo hel_FadeInit(void *pPalette, u8 PalMode, u8 ColMode, u8 PalNo);
extern u8 hel_FadePalette(TFadeInfo *pFadeInfo, s8 AmountPerCall);
extern void hel_FadeReset(TFadeInfo *pFadeInfo);


//
// ---[ SRAM functions ]----------------------------------------------------------------------------
//

extern u8 hel_RAMEntryExists(char *pIdentifier);


//
// ---[ Interrupt functions ]-----------------------------------------------------------------------
//

typedef void (*PInterruptProc)(void);

extern u8 hel_IntrIsEnabled(u32 InterruptType);
extern void hel_IntrStopHandler(u32 InterruptType);
extern void hel_IntrStopAll(void);
extern u32 hel_IntrGetType(void *InterruptProc);


//
// ---[ Bitmap-Mode functions ]---------------------------------------------------------------------
//

extern void hel_BmpClear(u16 Color);
extern u16 *hel_BmpGetBackBuffer(void);



#ifdef __cplusplus
   }
#endif

#endif /* HEL_H */


/* END OF FILE */

